import _ from 'underscore';

import defaultMapStateToProps from './mapStateToProps';
import defaultMapDispatchToProps from './mapDispatchToProps';
import defaultMergeProps from './mergeProps';
import mixin from './mixin';
import isDisplayComponent from './isDisplayComponent';

export default function connect(_mapStateToProps, _mapDispatchToProps, _mergeProps, _options) {

  var options = _options || {};

  return function (Component) {

    var mapStateToProps = _mapStateToProps || Component.prototype.mapStateToProps || defaultMapStateToProps;
    var mapDispatchToProps = _mapDispatchToProps || Component.prototype.mapDispatchToProps || defaultMapDispatchToProps;
    var mergeProps = _mergeProps || Component.prototype.mergeProps || defaultMergeProps;
    var store = options.store || Component.prototype.store;
    var componentInitialize = Component.prototype.initialize;
    var componentOnRender = Component.prototype.onRender;
    var componentOnDestroy = Component.prototype.onDestroy;
    var getInitialState = Component.prototype.getInitialState || function () {
      return {};
    };

    var connectMixin = _.defaults({}, {

      getInitialState: getInitialState,

      initialize: function initialize(_initOptions) {

        var initOptions = _initOptions || {};

        mixin.initialize.call(this, {
          mapStateToProps: mapStateToProps,
          mapDispatchToProps: mapDispatchToProps,
          mergeProps: mergeProps,
          store: store,
          props: initOptions.props
        });

        if (componentInitialize) {
          componentInitialize.apply(this, arguments);
        }
      },
      onRender: function onRender() {

        mixin.onRender.apply(this, arguments);

        if (componentOnRender) {
          componentOnRender.apply(this, arguments);
        }
      },
      onDestroy: function onDestroy() {

        mixin.onDestroy.apply(this, arguments);

        if (componentOnDestroy) {
          componentOnDestroy.apply(this, arguments);
        }
      }
    }, mixin);

    if (!isDisplayComponent(Component)) {
      connectMixin = _.omit(connectMixin, 'onRender');
    }

    return Component.extend(connectMixin);
  };
}